  
"
    __________________________________________________
    @Alejandro F. Reimondo 17/07/97
    This proyect sources are property of : Alejandro F. Reimondo
    Its use is NOT legal without written rights.
    Amenabar 3476 Piso: 2 Dto: B.
    Ciudad de Buenos Aires.
    Argentina.
    Tel: (54)-1-703-5586.
    mailTo:aleReimondo@smalltalking.net
    __________________________________________________
"
    (File pathName: 'WinInet.Pol') fileIn;close.
!

Object subclass: #Win32InternetObject
  instanceVariableNames: 
    ' handle winInetDLL '
  classVariableNames: ''
  poolDictionaries: ''  !

Win32InternetObject subclass: #InternetSession
  instanceVariableNames: 
    ' agent accessType proxy proxyBypass flags '
  classVariableNames: ''
  poolDictionaries: 
    ' WinInetConstants '.

  (Smalltalk at: #InternetSession) comment: '
	InternetSession objects model a connection between an application and internet API scheme.

	agent: A string that identifies the name of the application or entity calling the Internet APIs. This name is used as the user agent in the HTTP protocol.
	accessType: The type of access required.
		These are the valid values:
			InternetOpenTypePreconfig	Preconfigured (registry)
			InternetOpenTypeDirect	Direct to Internet
			InternetOpenTypeProxy	Through CERN proxy
	proxy: A string specifying the proxy server that is to be used if proxy access was specified. If a NULL value is supplied, the proxy information will be read from the registry.
	proxyBypass: An optional list of server addresses that may be bypassed when using proxy access. If a NULL value is supplied, the bypass list will be read from the registry.
	flags: Used to indicate various options such as caching and asynchronous behavior.
		The possible values include:
			InternetFlagOffline	Download operations on this handle are satisfied through the persistent cache only. If the item does not exist in the cache an appropriate error code is returned.
			InternetFlagAsync	Future operations on this handle may fail with ERROR_IO_PENDING. A status callback will be made with INTERNET_STATUS_REQUEST_COMPLETE.This callback will be on a thread other than the one for the original request. A status callback routine must be registered or the functions will be completed synchronously.

	@1997. Alejandro F. Reimondo'  !

InternetSession class instanceVariableNames: ' current '   !

Win32InternetObject subclass: #InternetSessionObject
  instanceVariableNames: 
    ' session '
  classVariableNames: ''
  poolDictionaries: 
    ' WinInetConstants '.

  (Smalltalk at: #InternetSessionObject) comment: '
	InternetSessionObject objects model objects in an InternetSession context.

	@1997. Alejandro F. Reimondo'  !

InternetSessionObject subclass: #HTTPRequestHeader
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: 
    ' OperatingSystemConstants WinInetConstants ' !

InternetSessionObject subclass: #InternetConnection
  instanceVariableNames: 
    ' server port user password service flags context '
  classVariableNames: ''
  poolDictionaries: 
    ' WinInetConstants '.

  (Smalltalk at: #InternetConnection) comment: '
	InternetConnection objects model a connection to a FTP, Gopher, or HTTP server.
	An InternetConnection can be used as an InternetSession for file & directory access (see #open documentation).

	server: a host of an Internet server. Alternately, the string with its IP number of the site in ASCII dotted-decimal format (for example, 11.0.1.45).
	port: The TCP/IP port on the server to connect to. If port is InvalidPortNumber (0), then the default port for the specified service is used.
	user: The user to log in. If nil, an appropriate default is used. For the FTP protocol, the default is anonymous.
	password: The password to use to log in. If both password and user are nil, the default anonymous password is used.
		In the case of FTP, the default anonymous password is the users email name.
		If password is nil (or an empty string) but user is not nil, a blank password is used.
	service: The type of service to access.
		This can be InternetServiceFtp, InternetServiceGopher, or InternetServiceHttp.
	flags: This specifies the flags specific to the service used.
		The possible values for these are:
			dwService	dwFlags supported	
			InternetServiceFtp	InternetConnectFlagPassive	Use passive mode in all data connections for this FTP session.
	context: An application-defined value that is used to identify the application context for the returned handle in callbacks.

	@1997. Alejandro F. Reimondo'    !

InternetConnection subclass: #FTPConnection
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: 
    ' OperatingSystemConstants WinInetConstants '    !

InternetConnection subclass: #GopherConnection
  instanceVariableNames: 
    ' locator '
  classVariableNames: ''
  poolDictionaries: 
    ' OperatingSystemConstants WinInetConstants '  !

InternetConnection subclass: #HTTPConnection
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: 
    ' OperatingSystemConstants WinInetConstants '   !

InternetSessionObject subclass: #InternetFile
  instanceVariableNames: 
    ' address headers flags context '
  classVariableNames: ''
  poolDictionaries: 
    ' OperatingSystemConstants WinInetConstants '.

  (Smalltalk at: #InternetFile) comment: '
	InternetFile objects model a handle to read a complete FTP, Gopher or HTTP Universal Resource Locator (URL).

	address: the url string.
	flags: This specifies the action flags.
		The possible values are:
			InternetFlagReload	Get the data from the wire even if it is locally cached.
			InternetFlagDontCache	Do not cache the data, either locally or in any gateways.
			InternetFlagRawData	By default, InternetOpenUrl returns HTML formatted for directories. This overrides the default to return the raw data (WIN32_FIND_DATA structures for FTP, and GOPHER_FIND_DATA structures for Gopher).
			InternetFlagSecure	This flag is applicable to http requests only. It requests secure transactions on the wire with Secure Sockets Layer or PCT
			InternetFlagExistingConnect	If possible, reuse the existing connections to the server for new requests generated by InternetOpenUrl instead of creating a new session for each request.
	context: An application-defined value that is used to identify the application context for the returned handle in callbacks.

	@1997. Alejandro F. Reimondo'    !

Dynalink32 variableByteSubclass: #WinInetDLL
  classVariableNames: ''
  poolDictionaries: 
    ' WinInetConstants ' !


!Win32InternetObject methods !
   
asParameter
	"Answer the receiver in a form suitable for passing as a parameter to a host procedure call."

    ^self handle asParameter!

close
	" Close the receiver. "

	| result |
	self isValid ifFalse: [ ^true ].
	result := self winInetDLL internetCloseHandle: self asParameter.
	handle := nil.
	result
	ifTrue: [ self doesNotNeedFinalization ]
	ifFalse: [ self osError ].
	^result!
   
finalize
	"If the receiver in not referenced by any object in the system and it has asked to receive this message so it can do final clean up.  Do it now."

	[ self release ] on: Error do: [].
	super finalize.!
  
getOption: dwOption
	"Queries an internet option on the receiver's handle (use a 1kb buffer).
	Returns buffer contents."

    | buffer |
    buffer := String new: 1024.
	^buffer copyFrom: 1 to: (self queryOption: dwOption buffer: buffer)!

handle
    "Returns the instance var of the receiver
     for external manipulation"

	handle isNil ifTrue: [ self initializeHandle ].
    ^handle!

initializeHandle
	" Private - Initializes the receiver's handle"

	self open.
	self isValid ifTrue: [ self needsFinalization ].!

initializeWinInetDLL
    " Private - Initialize winInetDLL with current winInet library"

	winInetDLL := WinInetDLL library.!

isValid
	"True when the receiver can be used."

	^handle notNil and: [ handle isValid ]!
 
lastResponseInfo
	"Returns aditional error description and server response."

	| bufferSize |
	bufferSize := self lastResponseInfo: nil.
	 ^self lastResponseInfo: (String new: bufferSize)!
   
lastResponseInfo: buffer
	" Private - Returns aditional error description and server response."

    | lpdwError lpdwBufferLength |
    lpdwError := ExternalLong fromInteger: 0.
	lpdwBufferLength := ExternalLong fromInteger: buffer size.
	(self winInetDLL
		internetGetLastResponseInfo: lpdwError asParameter
	    lpszBuffer: buffer
	    lpdwBufferLength: lpdwBufferLength asParameter)
	ifFalse: [ ^self osError ].

	^buffer isNil
	ifTrue: [ lpdwBufferLength asInteger ]
	ifFalse: [ buffer trimBlanks ]!
   
newStatusCallbackSending: selector to: aReceiver
	" Sets up a status callback for progrssive operations on the receiver.
	returns the last callback address (-1 if failed or 0 if none)."

    ^self winInetDLL
		internetSetStatusCallback: self asParameter
		sending: selector
		to: aReceiver!
   
OosErrorMessage: anInteger
        "Private - Answer a String which contains the appropriate error description for the host error identified by anInteger.
	lastResponseInfo will be appenden in the super message."

	^(super osErrorMessage: anInteger),' ', self lastResponseInfo!
   
open
	" Open the receiver. "

	self implementedBySubclass.!
  
privateHandle: aHandle
    "Sets the instance var of the receiver."

	self close."old handle"
    handle := aHandle!

queryOption: dwOption buffer: lpBuffer
	" Private - Queries an internet option on the receiver's handle."

    | long |
    long := ExternalLong fromInteger: lpBuffer size.
    ( self winInetDLL
		internetQueryOption: self asParameter
		dwOption: dwOption asParameter
		lpBuffer: lpBuffer asParameter
		lpdwBufferLength: long asParameter)
	ifFalse: [ ^self osError ].
    ^long asInteger!
 
release
	"Perform any actions required before the receiver becomes unreferenceable."

	self close.
	^super release!
 
setOption: dwOption to: lpBuffer
	" Private - Sets an internet option on the receiver's handle."

    | long |
    long := ExternalLong fromInteger: lpBuffer size.
    ( self winInetDLL
		internetSetOption: self asParameter
		dwOption: dwOption asParameter
		lpBuffer: lpBuffer asParameter
		dwBufferLength: long asParameter)
	ifFalse: [ ^self osError ].
    ^long asInteger!
  
winInetDLL
    "Returns the instance var of the receiver
     for external manipulation"

	winInetDLL isNil ifTrue: [ self initializeWinInetDLL ].
    ^winInetDLL!

winInetDLL: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    winInetDLL := anObject! !



!InternetSession class methods !
 
agent: agent
	accessType: accessType
	proxy: proxy
	proxyBypass: proxyBypass
	flags: flags
	"Returns an instance of the receiver initialized with this values.
	Use nil for default values."

	^self new
		agent: agent;
		accessType: accessType;
		proxy: proxy;
		proxyBypass: proxyBypass;
		flags: flags;
		yourself!

current
        "Answer the current open instance of the receiver class."
    current isNil ifTrue: [ current := self getCurrent ].
    ^current!

current: aConnection
        "Private - set the current instance of the receiver class."
    current := aConnection!
  
currentOrNil
        "Private - answer the current open instance of the receiver class if set,
        otherwise nil."
    ^current!
 
getCurrent
	"Private - get (open) an instance of the receiver."

    ^self new open! !



!InternetSession methods !
 
accessType
    "Returns the instance var of the receiver
     for external manipulation"

	accessType isNil ifTrue: [ self initializeAccessType ].
    ^accessType!

accessType: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    accessType := anObject!
   
agent
    "Returns the instance var of the receiver
     for external manipulation"

	agent isNil ifTrue: [ self initializeAgent ].
    ^agent!

agent: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    agent := anObject!
 
flags
    "Returns the instance var of the receiver
     for external manipulation"

	flags isNil ifTrue: [ self initializeFlags ].
    ^flags!

flags: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    flags := anObject!
 
initializeAccessType
    " Private - Initialize initializeAccessType"

	accessType := InternetOpenTypePreconfig"Direct".!

initializeAgent
    " Private - Initialize agent"

	agent := Smalltalk product,' ',Smalltalk version.!
   
initializeFlags
    " Private - Initialize flags"

	flags := 0.!
 
initializeProxy
    " Private - Initialize proxy"

	proxy := nil. "proxy read from registry."!
   
initializeProxyBypass
    " Private - Initialize proxyBypass"

	proxyBypass := nil. "proxyBypass read from registry."!
   
open
	" Open the receiver."

	| integer |
	integer := self winInetDLL
		internetOpen: self agent asParameter
	    dwAccessType: self accessType asParameter
	    lpszProxy: self proxy asParameter
	    lpszProxyBypass: self proxyBypass asParameter
	    dwFlags: self flags asParameter.
	self privateHandle: (ExternalHandle fromInteger: integer).
	self isValid ifFalse: [ ^self osError ].

	self class currentOrNil isNil
	ifTrue: [ self class current: self ].!
 
proxy
    "Returns the instance var of the receiver
     for external manipulation"

	proxy isNil ifTrue: [ self initializeProxy ].
    ^proxy!

proxy: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    proxy := anObject!
 
proxyBypass
    "Returns the instance var of the receiver
     for external manipulation"

	proxyBypass isNil ifTrue: [ self initializeProxyBypass ].
    ^proxyBypass!

proxyBypass: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    proxyBypass := anObject!
 
release
	"Release the receiver."

    self == self class currentOrNil ifTrue: [ self class current: nil ].
	self close.
	^super release.! !



!InternetSessionObject class methods !
  
on: aSession
	"Returns an instance of the receiver for working in aSession."

	^self new
		session: aSession;
		yourself! !



!InternetSessionObject methods !

initializeSession
	" Private - Initializes the receiver's session to current session."

	session := InternetSession current.!

session
    "Returns the instance var of the receiver
     for external manipulation"

	session isNil ifTrue: [ self initializeSession ].
    ^session!

session: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    session := anObject! !



!HTTPRequestHeader methods !
   
addHeaders: headers
    "Adds one or more HTTP request headers to the receiver."

    ^self addHeaders: headers modifiers: HttpAddreqFlagAdd!

addHeaders: headers modifiers: modifiers
    "Adds one or more HTTP request headers to the receiver."

    ^self winInetDLL
		httpAddRequestHeaders: self asParameter
		lpszHeaders: headers asParameter
		dwHeadersLength: headers size
		dwModifiers: modifiers asParameter!
   
queryInfo: level
    "Adds one or more HTTP request headers to the receiver."

	| buffer bufferSize long |
	bufferSize := self queryInfo: level buffer: nil.
	buffer := String new: bufferSize.
	^self queryInfo: level buffer: buffer!
   
queryInfo: level buffer: buffer
    "Return response or request headers from a HTTP request."

	| lpdwIndex lpdwBufferLength |
    lpdwIndex := ExternalLong fromInteger: 0.
    lpdwBufferLength := ExternalLong fromInteger: buffer size.
	( self winInetDLL
		httpQueryInfo: self asParameter
		dwInfoLevel: level asParameter
		lpBuffer: buffer asParameter
		lpdwBufferLength: lpdwBufferLength asParameter
		lpdwIndex: lpdwIndex asParameter
	) ifFalse: [ ^self osError ].

	buffer isNil ifTrue: [ "query for buffer size..."
		^lpdwBufferLength asInteger
	].

	^Array
		with: (buffer copyFrom: 1 to: lpdwBufferLength asInteger)
		with: lpdwIndex asInteger!
 
replaceHeaders: headers
    "Replace/removes one or more HTTP request headers to the receiver."

    ^self addHeaders: headers modifiers: HttpAddreqFlagReplace!
 
send
    "Sends the specified request to the HTTP server."

	^self send: nil optional: nil!
  
send: headers
    "Sends the specified request to the HTTP server."

	^self send: headers optional: nil!
 
send: headers optional: optional
    "Sends the specified request to the HTTP server."

	^self winInetDLL
		httpSendRequest: self asParameter
		lpszHeaders: headers asParameter
		dwHeadersLength: headers size asParameter
		lpOptional: optional asParameter
		dwOptionalLength: optional size asParameter! !



!InternetConnection class methods !
 
server: aServerURL
	" Returns an instance of the receiver for working on aServerURL. "

	^self new
		server: aServerURL;
		yourself! !



!InternetConnection methods !

context
    "Returns the instance var of the receiver
     for external manipulation"

	context isNil ifTrue: [ self initializeContext ].
    ^context!

context: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    context := anObject!
 
directoryNames
	"Returns aCollection with all direcotry names."

	| result |
	result := OrderedCollection new.
	self directoryNamesDo: [:each| result add: each ].
	^result!
  
directoryNamesDo: iterationBlock
    "Evaluate the <iterationBlock> once with the name of each directory contained within the receiver.
        Warnings:
        -   Modifications to the receiver's contents while executing this method may lead to missed or duplicated directories."

	self searchResultsDo: [:searchResult |
		searchResult isDirectory
			ifTrue: [iterationBlock value:searchResult entityName]]!

endSearch: aSearchResult
	"Private - Close the opened search handle, in order to free it."

	aSearchResult isNil ifTrue: [ ^self ].
    (self winInetDLL internetCloseHandle: aSearchResult searchHandle asParameter)
        ifFalse: [ self osNotification ]!

fileNamesDo: iterationBlock
    "Evaluate the <iterationBlock> once with the name of each file contained within the receiver.
        Warnings:
        -   Modifications to the receiver's contents while executing this method may lead to missed or duplicated files."

	self searchResultsDo: [:searchResult |
		searchResult isFile
			ifTrue: [iterationBlock value:searchResult entityName]]!
 
findNext: aSearchResult
	"Private - Find the next entry in the directory search; answer the findData structure, or nil if no next element."

    | findData |
    findData := Win32FindData new.
    ( self winInetDLL
		internetFindNextFile: aSearchResult searchHandle asParameter
		lpvFindData: findData asParameter )
        ifTrue: [ ^aSearchResult findData: findData ].
    KernelLibrary getLastError = 18   "ERROR_NO_MORE_FILES"
        ifTrue: [ ^nil ]
        ifFalse: [ ^self osError ]
!

flags
    "Returns the instance var of the receiver
     for external manipulation"

	flags isNil ifTrue: [ self initializeFlags ].
    ^flags!

flags: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    flags := anObject!
 
formatted: filterString
    "Answer a collection of arrays of file information for the receiver directory, filtered using filterString,
         e.g. '*.txt'.  Each array has four entries: file name, size, date/time and attributes."

    | answer anArray timeStamp timeStream |
    answer := OrderedCollection new.
    self
        searchResultsDo: [:searchResult |
            searchResult isFile
			ifTrue: [
				timeStream := WriteStream on: String new.
				[
				timeStamp := searchResult lastWriteTime.
				timeStream
					nextPutAll: timeStamp date printString;
					space;
					nextPutAll: timeStamp time printString.
				] on: Error do: [ timeStream := nil ].
				anArray := Array new: 5.
				anArray
					at: 1 put: searchResult entityName;
					at: 2 put: searchResult fileSize;
					at: 3 put: (timeStream notNil ifTrue: [ timeStream contents ] ifFalse: [String new]);
					at: 4 put: searchResult fileAttributes;
					at: 5 put: "This entry is only used to sort by date"
						(timeStamp notNil
							ifTrue: [timeStamp date asSeconds + timeStamp time asSeconds]
							ifFalse: [String new]).
				answer add: anArray
			]
		]
        onPattern: filterString.
    ^answer!
   
ftp
	" Sets the receiver's service to Ftp"

	self service: InternetServiceFtp.!
  
gopher
	" Sets the receiver's service to Gopher"

	self service: InternetServiceGopher.!
 
hasDirectoryNamed: directoryName
    "Answer <true> if the receiver contains a directory named <fileName>."

    | searchResult |
    [	searchResult := self findFirst: directoryName.
        ^searchResult notNil and: [ searchResult entity isDirectory ]
	] ensure: [ self endSearch: searchResult ]!
 
hasFileNamed: fileName
    "Answer <true> if the receiver contains a file named <fileName>."

    | searchResult |
    [	searchResult := self findFirst: fileName.
        ^searchResult notNil and: [ searchResult entity isFile ]
	] ensure: [ self endSearch: searchResult ]!
  
http
	" Sets the receiver's service to HTTP"

	self service: InternetServiceHttp.!
   
initializeContext
	" Private - Initializes the receiver's context"

	context := 0.!
  
initializeFlags
	" Private - Initializes the receiver's flags"

	flags := 0.!

initializePassword
	" Private - Initializes the receiver's password"

	password := nil.!
 
initializePort
	" Private - Initializes the receiver's port"

	port := InternetInvalidPortNumber.!
   
initializeServer
	" Private - Initializes the receiver's server to local host machine."

	server := 'LocalHost'.!

initializeService
	" Private - Initializes the receiver's service to HTTP"

	self http.!
 
initializeUser
	" Private - Initializes the receiver's user"

	user := nil.!
 
open
	" Open the receiver and sets it's handle a a valid session handle for server connection."

	| integer |
	integer := self winInetDLL
		internetConnect: self session asParameter
	    lpszServerName: self server asParameter
	    nServerPort: self port asParameter
	    lpszUserName: self user asParameter
	    lpszPassword: self password asParameter
	    dwService: self service asParameter
	    dwFlags: self flags asParameter
	    dwContext: self context asParameter.
	self privateHandle: (ExternalHandle fromInteger: integer).
	self isValid ifFalse: [ ^self osError ].!
   
password
    "Returns the instance var of the receiver
     for external manipulation"

	password isNil ifTrue: [ self initializePassword ].
    ^password!

password: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    password := anObject!
   
port
    "Returns the instance var of the receiver
     for external manipulation"

	port isNil ifTrue: [ self initializePort ].
    ^port!

port: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    port := anObject!
   
release
	"Release the receiver."

	self close.
	^super release.!

searchResultsDo: iterationBlock
    "Private - Raw traversal of directory contents..."

	^self searchResultsDo: iterationBlock onPattern: '*.*'!
 
searchResultsDo: iterationBlock onPattern: filterString
    "Private - Raw traversal of directory contents ... searchResult contains interesting information"

    | searchResult |
    searchResult := self findFirst: filterString.
	searchResult isNil ifTrue: [ ^self ].
    [
		searchResult isInteresting ifTrue: [ iterationBlock value: searchResult ].

		[ (self findNext: searchResult) notNil ]
		whileTrue: [
			searchResult isInteresting ifTrue: [ iterationBlock value: searchResult ]
		].
	] ensure: [ self endSearch: searchResult ].!
 
server
    "Returns the instance var of the receiver
     for external manipulation"

	server isNil ifTrue: [ self initializeServer ].
    ^server!

server: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    server := anObject!
   
service
    "Returns the instance var of the receiver
     for external manipulation"

	service isNil ifTrue: [ self initializeService ].
    ^service!

service: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    service := anObject!
 
user
    "Returns the instance var of the receiver
     for external manipulation"

	user isNil ifTrue: [ self initializeUser ].
    ^user!

user: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    user := anObject! !



!FTPConnection methods !
 
command: command
	response: aBoolean
	transferType: transferType

    "Issues an arbitrary command on the FTP server."

	( self winInetDLL
		ftpCommand: self asParameter
		fExpectResponse: aBoolean asParameter
		dwFlags: transferType asParameter
		lpszCommand: command asParameter
		dwContext: self context asParameter
	) ifFalse: [ ^self osError ].!
  
createDirectory: lpszDirectory
	" Creates a new directory on the server.
	Returns true if successful. "

	^self winInetDLL
		ftpCreateDirectory: self asParameter
		lpszDirectory: lpszDirectory!
 
currentDirectory
	" Returns the current directory pathName if successful or nil. "

	| success lpszCurrentDirectory lpdwCurrentDirectorySize |
	lpszCurrentDirectory := String new: MaxPath.
    lpdwCurrentDirectorySize := ExternalLong fromInteger: lpszCurrentDirectory size.
	success := self winInetDLL
		ftpGetCurrentDirectory: self asParameter
		lpszCurrentDirectory: lpszCurrentDirectory
		lpdwCurrentDirectory: lpdwCurrentDirectorySize.
	^success ifTrue: [ lpszCurrentDirectory trimBlanks ]!

currentDirectory: pathName
	" Sets the current working directory.
	Returns true if successful. "

	^self winInetDLL
		ftpSetCurrentDirectory: self asParameter
		lpszDirectory: pathName asParameter!
 
deleteFile: lpszFileName
	" Deletes a file on the server. Returns true if successful. "

	^self winInetDLL
		ftpDeleteFile: self asParameter
		lpszFileName: lpszFileName!
 
findFirst: aString
    "Private - Search in the current directory for an entry (File or directory) matching the filespec aString;
	answer an Array containing the findData structure and a search handle if found, otherwise nil.
	The caller must call #endSearch: with the search handle as an argument when done searching."

    | aFindData aSearchHandle error |
    aFindData := Win32FindData new.
    aSearchHandle := self winInetDLL
		ftpFindFirstFile: self asParameter
		lpszSearchFile: aString asParameter
		lpFindFileData: aFindData asParameter
		dwFlags: 0 dwContext: self context.
    aSearchHandle = InvalidHandleValue
	ifTrue: [
		error := KernelLibrary getLastError.
            "ignore the following errors:
            ERROR_FILE_NOT_FOUND
            ERROR_PATH_NOT_FOUND
            ERROR_INVALID_DRIVE
            ERROR_NO_MORE_FILES
            ERROR_NOT_READY"
            (#(2 3 15 18 21) includes: error) ifFalse: [self osNotification].
		^nil
	].
	aSearchHandle := ExternalHandle fromInteger: aSearchHandle.
	aSearchHandle isValid ifFalse: [ ^nil ].
    ^FileSearchResults
            directory: self
            searchHandle: aSearchHandle
            findData: aFindData!
 
getFile: remoteFileName newFile: localFileName
	"Retrieves a file from the server and stores it under the specified filename, creating a new local file. "

    ^self
		getFile: remoteFileName
		newFile: localFileName
		failIfExists: false!
   
getFile: remoteFileName
	newFile: localFileName
	failIfExists: failIfExists

	"Retrieves a file from the server and stores it under the specified filename, creating a new local file. "

    ^self
		getFile: remoteFileName
		newFile: localFileName
		failIfExists: failIfExists
		attributes: 0!
  
getFile: remoteFileName
	newFile: localFileName
	failIfExists: failIfExists
	attributes: attributes

	"Retrieves a file from the server and stores it under the specified filename, creating a new local file. "

    ^self
		getFile: remoteFileName
		newFile: localFileName
		failIfExists: failIfExists
		attributes: attributes
		transferType: FtpTransferTypeBinary!
 
getFile: remoteFileName
	newFile: localFileName
	failIfExists: failIfExists
	attributes: attributes
	transferType: transferType
	"Retrieves a file from the server and stores it under the specified filename, creating a new local file. "

    ^self winInetDLL
		ftpGetFile: self asParameter
	    lpszRemoteFile: remoteFileName asParameter
    	lpszNewFile: localFileName asParameter
	    fFailIfExists: failIfExists asParameter
    	dwFlagsAndAttributes: attributes asParameter
	    dwFlags: transferType asParameter
    	dwContext: self context asParameter!
  
initializeService
	" Private - Initializes the receiver's service to FTP"

	self ftp.!
   
openFile: fileName
    "Initiates access to a remote file for writing."

   ^self
		openFile: fileName
		access: GenericWrite!
 
openFile: fileName access: access
    "Initiates access to a remote file for writing or reading."

   ^self
		openFile: fileName
		access: access
		transferType: FtpTransferTypeBinary!
  
openFile: fileName access: access transferType: transferType
    "Initiates access to a remote file for writing or reading."

    | aHandle |
    aHandle := self winInetDLL
        ftpOpenFile: self asParameter
        lpszFileName: fileName asParameter
        dwAccess: access asParameter
        dwFlags: transferType asParameter
        dwContext: self context asParameter.
    aHandle = 0 ifTrue: [ ^self osError ].
    ^InternetFile new
        address: fileName;
        session: self;
        privateHandle: aHandle;
        yourself!
   
openFileReadOnly: fileName
    "Initiates access to a remote file for reading."

   ^self
		openFile: fileName
		access: GenericRead!
  
putFile: localFileName into: remoteFileName
	"Stores a file into the server in binary (default) mode."

    ^self
		putFile: localFileName
		into: remoteFileName
		transferType: FtpTransferTypeBinary!
  
putFile: localFileName into: remoteFileName transferType: transferType
	"Stores a file into the server."

    ^self winInetDLL
		ftpPutFile: self asParameter
	    lpszLocalFile: localFileName asParameter
    	lpszNewRemoteFile: remoteFileName asParameter
	    dwFlags: transferType asParameter
    	dwContext: self context asParameter!
 
removeDirectory: lpszDirectory
	" Removes the directory on the server.
	Returns true if successful. "

	^self winInetDLL
		ftpRemoveDirectory: self asParameter
		lpszDirectory: lpszDirectory!
   
renameFile: currentFileName to: newFileName
	"Renames a file in the server."

    ^self winInetDLL
		ftpRenameFile: self asParameter
		lpszExisting: currentFileName asParameter
		lpszNew: newFileName asParameter! !



!GopherConnection methods !
 
findFirst: aString
    "Private - Search in the current directory for an entry (File or directory) matching the filespec aString;
	answer an Array containing the findData structure and a search handle if found, otherwise nil.
	The caller must call #endSearch: with the search handle as an argument when done searching."

    | aFindData aSearchHandle error |
    aFindData := self winInetDLL structureNamed: 'GOPHER_FIND_DATA'.
    aSearchHandle := self winInetDLL
		gopherFindFirstFile: self asParameter
		lpszLocator: self locator asParameter
		lpszSearchString: aString asParameter
		lpFindData: aFindData asParameter
		dwFlags: 0 dwContext: self context.
    aSearchHandle = InvalidHandleValue
	ifTrue: [
		error := KernelLibrary getLastError.
            "ignore the following errors:
            ERROR_FILE_NOT_FOUND
            ERROR_PATH_NOT_FOUND
            ERROR_INVALID_DRIVE
            ERROR_NO_MORE_FILES
            ERROR_NOT_READY"
            (#(2 3 15 18 21) includes: error) ifFalse: [self osNotification].
		^nil
	].
	aSearchHandle := ExternalHandle fromInteger: aSearchHandle.
	aSearchHandle isValid ifFalse: [ ^nil ].
    ^FileSearchResults
            directory: self
            searchHandle: (ExternalHandle fromInteger: aSearchHandle)
            findData: aFindData "to be revised for proper funcionality"!

getAttribute: attributeNamesString
	"Retrieve specific attribute information from the server.
	attributeNamesString is a space-delimited string specifying the names of attributes to return.
	If attributeNamesString is nil all names will be returned."

	^self getAttribute: attributeNamesString sending: nil to: nil!

getAttribute: attributeNamesString sending: aSelector to: aReceiver
	"Retrieve specific attribute information from the server.
	attributeNamesString is a space-delimited string specifying the names of attributes to return.
	if aSelector is not nil aReceiver will be notified via callback for each attribute with aSelector.
	If attributeNamesString is nil all names will be returned."

	| callBack lpdwCharactersReturned buffer |
	callBack := aSelector notNil ifTrue: [
		self winInetDLL newCallBack: #gopherAttributeEnumerator sending: aSelector to: aReceiver
	].
	lpdwCharactersReturned := ExternalLong fromInteger: 0.
	buffer := String new: MinGopherAttributeLength * 5 // 3.
	( self winInetDLL
		gopherGetAttribute: self asParameter
		lpszLocator: self locator asParameter
		lpszAttributeName: attributeNamesString asParameter
		lpBuffer: buffer
		dwBufferLength: buffer size asParameter
		lpdwCharactersReturned: lpdwCharactersReturned
		lpfnEnumerator: callBack asParameter
		dwContext: self context asParameter )
	ifFalse: [ ^self osError ].
	^buffer copyfrom: 1 to: lpdwCharactersReturned asInteger!

host: hostAddress
	port: serverPort
    display: lpszDisplayString
    selector: lpszSelectorString
    type: dwGopherType
	" Sets the receiver's Gopher or Gopher+ locator from arguments."

	| lpdwBufferLength buffer |
    lpdwBufferLength := self
		host: hostAddress
		port: serverPort
	    display: lpszDisplayString
	    selector: lpszSelectorString
	    type: dwGopherType
		locator: nil.
	buffer := String new: lpdwBufferLength + 1.
    lpdwBufferLength := self
		host: hostAddress
		port: serverPort
	    display: lpszDisplayString
	    selector: lpszSelectorString
	    type: dwGopherType
		locator: buffer.
	self locator: (buffer copyFrom: 1 to: lpdwBufferLength).
	^self locator!

host: hostAddress
	port: serverPort
    display: lpszDisplayString
    selector: lpszSelectorString
    type: dwGopherType
	locator: lpszLocator
	" Private - Fill lpszLocator with Gopher or Gopher+ locator from arguments (returns the number of bytes filled)."

	| lpdwBufferLength |
    lpdwBufferLength := ExternalLong fromInteger: lpszLocator size.
    ( self winInetDLL
		gopherCreateLocator: hostAddress asParameter
		nServerPort: serverPort asParameter
		lpszDisplayString: lpszDisplayString asParameter
		lpszSelectorString: lpszSelectorString asParameter
		dwGopherType: dwGopherType asParameter
		lpszLocator: lpszLocator
		lpdwBufferLength: lpdwBufferLength asParameter )
	ifFalse: [ ^self osError ].
	^lpdwBufferLength asInteger!
  
initializeService
	" Private - Initializes the receiver's service to FTP"

	self gopher.!

locator
    "Returns the instance var of the receiver
     for external manipulation"
    ^locator!
  
locator: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    locator := anObject!
 
locatorType
    "Returns the current locator type."

	| lpdwGopherType |
	lpdwGopherType := ExternalLong fromInteger: 0.
    ( self winInetDLL
		gopherGetLocatorType: self locator asParameter
		lpdwGopherType: lpdwGopherType asParameter )
	ifFalse: [ ^self osError ].
	^lpdwGopherType asInteger!

open: fileName view: aViewName flags: internetFlags
    "Initiates access to a remote file for reading."

	^self
		locator: fileName;
		openView: aViewName
		flags: internetFlags!
   
openFile: fileName
    "Initiates access to a remote file for reading."

	^self
		locator: fileName;
		openView: nil flags: 0!
 
openFile: fileName flags: internetFlags
    "Initiates access to a remote file for reading."

	^self
		locator: fileName;
		openView: nil
		flags: internetFlags!
 
openView: aViewName flags: internetFlags
    "Initiates access to a remote file for reading."

    | aHandle |
    aHandle := self winInetDLL
		gopherOpenFile: self asParameter
		lpszLocator: self locator asParameter
		lpszView: aViewName asParameter
        dwFlags: internetFlags asParameter
        dwContext: self context asParameter.
    aHandle = 0 ifTrue: [ ^self osError ].
    ^InternetFile new
        address: self locator;
        session: self;
        privateHandle: aHandle;
        yourself! !



!HTTPConnection methods !
   
openRequest: verb
	objectName: objectName
	flags: openFlags
    "Opens an HTTP request handle."

   	^self
		openRequest: verb
		objectName: objectName
		version: nil "HTTP/1.0"
		referrer: nil
		acceptTypes: nil "only text/* will be accepted"
		flags: openFlags!
  
openRequest: verb
	objectName: objectName
	version: version
	acceptTypes: acceptTypes
	flags: openFlags
    "Opens an HTTP request handle."

   	^self
		openRequest: verb
		objectName: objectName
		version: version
		referrer: nil
		acceptTypes: acceptTypes
		flags: openFlags!
  
openRequest: verb
	objectName: objectName
	version: version
	referrer: referrer
	acceptTypes: acceptTypes
	flags: openFlags
    "Opens an HTTP request handle."

    | aHandle |
    aHandle := self winInetDLL
		httpOpenRequest: self asParameter
		lpszVerb: verb asParameter
		lpszObjectName: objectName asParameter
		lpszVersion: version asParameter
		lpszReferrer: referrer asParameter
		lplpszAcceptTypes: acceptTypes asParameter
		dwFlags: openFlags asParameter
		dwContext: self context asParameter.
    aHandle = 0 ifTrue: [ ^self osError ].
    ^HTTPRequestHeader new
        address: objectName;
        session: self;
        privateHandle: aHandle;
        yourself! !



!InternetFile methods !
  
addFlagDontCache
	"Set the InternetFlagDontCache flag."

	self flags: self flags | InternetFlagDontCache!

addFlagExistingConnect
	"Set the InternetFlagExistingConnect flag."

	self flags: self flags | InternetFlagExistingConnect!
  
addFlagRawData
	"Set the InternetFlagRawData flag."

	self flags: self flags | InternetFlagRawData!
  
addFlagReload
	"Set the InternetFlagReload flag."

	self flags: self flags | InternetFlagReload!
 
addFlagSecure
	"Set the InternetFlagSecure flag."

	self flags: self flags | InternetFlagSecure!
 
address
    "Returns the instance var of the receiver
     for external manipulation"

	address isNil ifTrue: [ self initializeAddress ].
    ^address!

address: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    address := anObject!
 
context
    "Returns the instance var of the receiver
     for external manipulation"

	context isNil ifTrue: [ self initializeContext ].
    ^context!

context: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    context := anObject!
 
flags
    "Returns the instance var of the receiver
     for external manipulation"

	flags isNil ifTrue: [ self initializeFlags ].
    ^flags!

flags: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    flags := anObject!
 
headers
    "Returns the instance var of the receiver
     for external manipulation"

	headers isNil ifTrue: [ self initializeHeaders ].
    ^headers!

headers: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    headers := anObject!
 
initializeAddress
	" Private - Initializes the receiver's address"

	address := 'default.htm'. "default address settings"!
   
initializeContext
	" Private - Initializes the receiver's context"

	context := 0.!
  
initializeFlags
	" Private - Initializes the receiver's flags"

	flags := 0.!

initializeHeaders
	" Private - Initializes the receiver's headers"

	headers := nil. "no default headers."!
  
next
	" Try to read the next character from the receiver's data.
	If success returns it else returns nil. "

	| string |
    string := self next: 1.
	^string isEmpty ifFalse: [ string first ]!
  
next: count
	" Try to read the next count bytes from the receiver.
	aString of count bytes will be returned or a smaller string if EOF encountered. "

    ^self next: count into: (String new: count)!
 
next: count into: buffer
	" Private - Try to read the next count bytes from the receiver.
	aString of count bytes will be returned or a smaller string if EOF encountered. "

    | read |
	read := self readInto: buffer.
	^buffer copyFrom: 1 to: read!
 
nextPut: aCharacter
	"Write aCharacter to the receiver file."

	^self nextPutAll: (String with: aCharacter)!
 
nextPutAll: aString
	"Write bytes of aString to the receiver file."

	^self write: aString size from: aString!
   
open
	" Open the receiver."

	| integer |
	integer := self winInetDLL
		internetOpenUrl: self session asParameter
	    lpszUrl: self address asParameter
		lpszHeaders: self headers asParameter
		dwHeadersLength: self headers size
	    dwFlags: self flags asParameter
		dwContext: self context asParameter.
	self privateHandle: (ExternalHandle fromInteger: integer).
	self isValid ifFalse: [ ^self osError ].!

position
    " Answer the current file position. "

	^self setFilePointer: 0 moveMethod: FileCurrent!

position: anInteger
    "Private - Position the receiver to anInteger."

	^self setFilePointer: anInteger moveMethod: FileBegin!
 
queryDataAvailable
	"Returns the bytes that can be read from the receiver inmediatelly."

    | long |
    long := ExternalLong fromInteger: 0.
    ( self winInetDLL
		internetQueryDataAvailable: self asParameter
		lpdwNumberOfBytesAvailable: long asParameter
		dwFlags: 0 asParameter
		dwContext: self context asParameter)
	ifFalse: [ ^self osError ].
    ^long asInteger!

read: bytes into: aString
	"Private - Read the receiver's data into aString.  Answer the number of bytes read. "

    | long |
    long := ExternalLong fromInteger: 0.
    ( self winInetDLL
		internetReadFile: self asParameter
		lpBuffer: aString
		dwNumberOfBytesToRead: bytes
		lpdwNumberOfBytesRead: long asParameter )
	ifFalse: [ ^self osError ].
    ^long asInteger!
  
readBuffer: aString atPosition: anInteger
	"Read a page or less (if at end of file) at position anInteger modulo aString size from the receiver file into aString.
	Answer the number bytes read."

    ^self readInto: aString
        atPage: anInteger // aString size
        pageSize: aString size!
 
readInto: aString
	" Read the receiver's data into aString.  Answer the number of bytes read. "

    ^self read: aString size into: aString!
 
readInto: aString atPage: anInteger pageSize: sizeInteger
	"Private - Read a page (sizeInteger bytes) or less (if at end of file) at page number anInteger from the receiver file into aString.
	Answer the number of bytes read.
	aString size must be >= sizeInteger."

    ^self
		position: anInteger * sizeInteger;
		read: sizeInteger into: aString!
  
readInto: aString atPosition: anInteger
	"Read a page or less (if at end of file) at position anInteger modulo aString size from the receiver file into aString.
	Answer the number bytes read."

    ^self readInto: aString
        atPage: anInteger // aString size
        pageSize: aString size!
   
setEndOfFileTo: anInteger
    "Truncate or extend the end-of-file position of the receiver."

    self notImplementedYet!

setFilePointer: anInteger moveMethod: moveMethod
    "Private - Sets a file position for InternetReadFile.
	It is a synchronous call, however subsequent calls to InternetReadFile may block or return pending if the data is not available from the cache and the server does not support random access."

	| result |
    ( result := self winInetDLL
        internetSetFilePointer: self asParameter
        lDistanceToMove: anInteger asParameter
		pReserved: nil asParameter
        dwMoveMethod: moveMethod asParameter) = 16rFFFFFFFF "-1"
            ifTrue: [ ^self osError ].
	^result!
   
url
    "Returns the receiver's address."

    ^self address!

url: aString
    "Sets the receiver's address."

    ^self address: aString!
 
write: aString
	"Write bytes of aString to the receiver file."

	^self write: aString size from: aString!

write: size from: aString
	"Write size bytes of aString to the receiver file."

    | long |
    long := ExternalLong fromInteger: 0.
    ( self winInetDLL
		internetWriteFile: self asParameter
		lpBuffer: aString
		dwNumberOfBytesToWrite: size
		lpdwNumberOfBytesWritten: long asParameter )
	ifFalse: [ ^self osError ].
    ^long asInteger!

writeFrom: aString toPage: anInteger for: size pageSize: sizeInteger
	"Private - Write size bytes of aString to the receiver file at page number anInteger.
	Page size is sizeInteger."

	^self
		position: anInteger * sizeInteger;
		write: size from: aString!
 
writeFrom: aString toPosition: anInteger for: size
        "Write size bytes of aString to the receiver
         file at position anInteger modulo aString size."
    ^self writeFrom: aString
        toPage: anInteger // aString size
        for: size
        pageSize: aString size! !



!WinInetDLL class methods !
   
callbackTable
    " Private - Returns the callback shape table for defining callback handlers."

    ^#(
    "(callBackName (arguments...) returnType callingConvertion)"

    "typedef
        VOID
        (CALLBACK * INTERNET_STATUS_CALLBACK)(
            IN HINTERNET hInternet,
            IN DWORD dwContext,
            IN DWORD dwInternetStatus,
            IN LPVOID lpvStatusInformation OPTIONAL,
            IN DWORD dwStatusInformationLength
        );
    "
    (internetStatusCallback (handle ulong ulong ulong ulong) none api)

    "
    typedef
    BOOL
    (CALLBACK * GOPHER_ATTRIBUTE_ENUMERATOR)(
        LPGOPHER_ATTRIBUTE_TYPE lpAttributeInfo,
        DWORD dwError
        );
    "
    (gopherAttributeEnumerator (ulong ulong) boolean api)

    )!
  
fileName
    "Private - Answer the receiver's file name."

     ^'WinInet.Dll'!
  
initializeConstants
   "Private - Initializes the constants for the interface...
        WinInetDLL initializeConstants.
    "

    WinInetConstants size = 0 ifTrue: [
        WinInetConstants := Dictionary new
    ].

  WinInetConstants
    at: 'CernProxyInternetAccess' put: 3;
    at: 'ErrorFtpDropped' put: 12111;
    at: 'ErrorFtpTransferInProgress' put: 12110;
    at: 'ErrorGopherAttributeNotFound' put: 12137;
    at: 'ErrorGopherDataError' put: 12132;
    at: 'ErrorGopherEndOfData' put: 12133;
    at: 'ErrorGopherIncorrectLocatorType' put: 12135;
    at: 'ErrorGopherInvalidLocator' put: 12134;
    at: 'ErrorGopherNotFile' put: 12131;
    at: 'ErrorGopherNotGopherPlus' put: 12136;
    at: 'ErrorGopherProtocolError' put: 12130;
    at: 'ErrorGopherUnknownLocator' put: 12138;
    at: 'ErrorHttpDownlevelServer' put: 12151;
    at: 'ErrorHttpHeaderAlreadyExists' put: 12155;
    at: 'ErrorHttpHeaderNotFound' put: 12150;
    at: 'ErrorHttpInvalidHeader' put: 12153;
    at: 'ErrorHttpInvalidQueryRequest' put: 12154;
    at: 'ErrorHttpInvalidServerResponse' put: 12152;
    at: 'ErrorInternetBadOptionLength' put: 12010;
    at: 'ErrorInternetBadRegistryParameter' put: 12022;
    at: 'ErrorInternetCannotConnect' put: 12029;
    at: 'ErrorInternetConnectionAborted' put: 12030;
    at: 'ErrorInternetConnectionReset' put: 12031;
    at: 'ErrorInternetExtendedError' put: 12003;
    at: 'ErrorInternetForceRetry' put: 12032;
    at: 'ErrorInternetHandleExists' put: 12036;
    at: 'ErrorInternetIncorrectFormat' put: 12027;
    at: 'ErrorInternetIncorrectHandleState' put: 12019;
    at: 'ErrorInternetIncorrectHandleType' put: 12018;
    at: 'ErrorInternetIncorrectPassword' put: 12014;
    at: 'ErrorInternetIncorrectUserName' put: 12013;
    at: 'ErrorInternetInternalError' put: 12004;
    at: 'ErrorInternetInvalidOperation' put: 12016;
    at: 'ErrorInternetInvalidOption' put: 12009;
    at: 'ErrorInternetInvalidUrl' put: 12005;
    at: 'ErrorInternetItemNotFound' put: 12028;
    at: 'ErrorInternetLoginFailure' put: 12015;
    at: 'ErrorInternetMixedSecurity' put: 12034;
    at: 'ErrorInternetNameNotResolved' put: 12007;
    at: 'ErrorInternetNoCallback' put: 12025;
    at: 'ErrorInternetNoContext' put: 12024;
    at: 'ErrorInternetNoDirectAccess' put: 12023;
    at: 'ErrorInternetNotProxyRequest' put: 12020;
    at: 'ErrorInternetOperationCancelled' put: 12017;
    at: 'ErrorInternetOptionNotSettable' put: 12011;
    at: 'ErrorInternetOutOfHandles' put: 12001;
    at: 'ErrorInternetProtocolNotFound' put: 12008;
    at: 'ErrorInternetRegistryValueNotFound' put: 12021;
    at: 'ErrorInternetRequestPending' put: 12026;
    at: 'ErrorInternetShutdown' put: 12012;
    at: 'ErrorInternetSslCertCnInvalid' put: 12035;
    at: 'ErrorInternetTimeout' put: 12002;
    at: 'ErrorInternetUnrecognizedScheme' put: 12006;
    at: 'ErrorInternetZoneCrossing' put: 12033;
    at: 'FlagsErrorUiFilterForErrors' put: 1;
    at: 'FlagsErrorUiFlagsChangeOptions' put: 2;
    at: 'FlagsErrorUiFlagsGenerateData' put: 4;
    at: 'FtpTransferTypeAscii' put: 1;
    at: 'FtpTransferTypeBinary' put: 2;
    at: 'FtpTransferTypeMask' put: 3;
    at: 'FtpTransferTypeUnknown' put: 0;
    at: 'GatewayInternetAccess' put: 2;
    at: 'GopherAbstractAttribute' put: 'Abstract';
    at: 'GopherAbstractCategory' put: '+ABSTRACT';
    at: 'GopherAdminAttribute' put: 'Admin';
    at: 'GopherAdminCategory' put: '+ADMIN';
    at: 'GopherAttributeIdAbstract' put: 2882325526;
    at: 'GopherAttributeIdAdmin' put: 2882325514;
    at: 'GopherAttributeIdAll' put: 2882325513;
    at: 'GopherAttributeIdBase' put: 2882325504;
    at: 'GopherAttributeIdGeog' put: 2882325522;
    at: 'GopherAttributeIdLocation' put: 2882325521;
    at: 'GopherAttributeIdModDate' put: 2882325515;
    at: 'GopherAttributeIdOrg' put: 2882325520;
    at: 'GopherAttributeIdProvider' put: 2882325524;
    at: 'GopherAttributeIdRange' put: 2882325518;
    at: 'GopherAttributeIdScore' put: 2882325517;
    at: 'GopherAttributeIdSite' put: 2882325519;
    at: 'GopherAttributeIdTimezone' put: 2882325523;
    at: 'GopherAttributeIdTreewalk' put: 2882325528;
    at: 'GopherAttributeIdTtl' put: 2882325516;
    at: 'GopherAttributeIdUnknown' put: 2882325529;
    at: 'GopherAttributeIdVersion' put: 2882325525;
    at: 'GopherAttributeIdView' put: 2882325527;
    at: 'GopherCategoryIdAbstract' put: 2882325509;
    at: 'GopherCategoryIdAdmin' put: 2882325507;
    at: 'GopherCategoryIdAll' put: 2882325505;
    at: 'GopherCategoryIdAsk' put: 2882325511;
    at: 'GopherCategoryIdInfo' put: 2882325506;
    at: 'GopherCategoryIdUnknown' put: 2882325512;
    at: 'GopherCategoryIdVeronica' put: 2882325510;
    at: 'GopherCategoryIdViews' put: 2882325508;
    at: 'GopherGeogAttribute' put: 'Geog';
    at: 'GopherInfoCategory' put: '+INFO';
    at: 'GopherLocationAttribute' put: 'Loc';
    at: 'GopherModDateAttribute' put: 'Mod-Date';
    at: 'GopherOrgAttribute' put: 'Org';
    at: 'GopherProviderAttribute' put: 'Provider';
    at: 'GopherRangeAttribute' put: 'Score-range';
    at: 'GopherScoreAttribute' put: 'Score';
    at: 'GopherSiteAttribute' put: 'Site';
    at: 'GopherTimezoneAttribute' put: 'TZ';
    at: 'GopherTreewalkAttribute' put: 'treewalk';
    at: 'GopherTtlAttribute' put: 'TTL';
    at: 'GopherTypeAsk' put: 1073741824;
    at: 'GopherTypeBinary' put: 512;
    at: 'GopherTypeBitmap' put: 16384;
    at: 'GopherTypeCalendar' put: 524288;
    at: 'GopherTypeCso' put: 4;
    at: 'GopherTypeDirectory' put: 2;
    at: 'GopherTypeDosArchive' put: 32;
    at: 'GopherTypeError' put: 8;
    at: 'GopherTypeFileMask' put: 2093681;
    at: 'GopherTypeGif' put: 4096;
    at: 'GopherTypeGopherPlus' put: 2147483648;
    at: 'GopherTypeHtml' put: 131072;
    at: 'GopherTypeImage' put: 8192;
    at: 'GopherTypeIndexServer' put: 128;
    at: 'GopherTypeInline' put: 1048576;
    at: 'GopherTypeMacBinhex' put: 16;
    at: 'GopherTypeMovie' put: 32768;
    at: 'GopherTypePdf' put: 262144;
    at: 'GopherTypeRedundant' put: 1024;
    at: 'GopherTypeSound' put: 65536;
    at: 'GopherTypeTelnet' put: 256;
    at: 'GopherTypeTextFile' put: 1;
    at: 'GopherTypeTn3270' put: 2048;
    at: 'GopherTypeUnixUuencoded' put: 64;
    at: 'GopherTypeUnknown' put: 536870912;
    at: 'GopherVeronicaCategory' put: '+VERONICA';
    at: 'GopherVersionAttribute' put: 'Version';
    at: 'GopherViewAttribute' put: 'View';
    at: 'GopherViewsCategory' put: '+VIEWS';
    at: 'HttpAddreqFlagAdd' put: 536870912;
    at: 'HttpAddreqFlagAddIfNew' put: 268435456;
    at: 'HttpAddreqFlagCoalesce' put: 1073741824;
    at: 'HttpAddreqFlagReplace' put: 2147483648;
    at: 'HttpAddreqFlagsMask' put: 4294901760;
    at: 'HttpAddreqIndexMask' put: 65535;
    at: 'HttpMajorVersion' put: 1;
    at: 'HttpMinorVersion' put: 0;
    at: 'HttpQueryAccept' put: 24;
    at: 'HttpQueryAcceptCharset' put: 25;
    at: 'HttpQueryAcceptEncoding' put: 26;
    at: 'HttpQueryAcceptLanguage' put: 27;
    at: 'HttpQueryAcceptRanges' put: 42;
    at: 'HttpQueryAllow' put: 7;
    at: 'HttpQueryAuthorization' put: 28;
    at: 'HttpQueryConnection' put: 23;
    at: 'HttpQueryContentDescription' put: 4;
    at: 'HttpQueryContentEncoding' put: 29;
    at: 'HttpQueryContentId' put: 3;
    at: 'HttpQueryContentLanguage' put: 6;
    at: 'HttpQueryContentLength' put: 5;
    at: 'HttpQueryContentTransferEncoding' put: 2;
    at: 'HttpQueryContentType' put: 1;
    at: 'HttpQueryCost' put: 15;
    at: 'HttpQueryCustom' put: 65535;
    at: 'HttpQueryDate' put: 9;
    at: 'HttpQueryDerivedFrom' put: 14;
    at: 'HttpQueryExpires' put: 10;
    at: 'HttpQueryFlagCoalesce' put: 268435456;
    at: 'HttpQueryFlagNumber' put: 536870912;
    at: 'HttpQueryFlagRequestHeaders' put: 2147483648;
    at: 'HttpQueryFlagSystemtime' put: 1073741824;
    at: 'HttpQueryForwarded' put: 30;
    at: 'HttpQueryFrom' put: 31;
    at: 'HttpQueryHeaderMask' put: 0;
    at: 'HttpQueryIfModifiedSince' put: 32;
    at: 'HttpQueryLastModified' put: 11;
    at: 'HttpQueryLink' put: 16;
    at: 'HttpQueryLocation' put: 33;
    at: 'HttpQueryMax' put: 42;
    at: 'HttpQueryMessageId' put: 12;
    at: 'HttpQueryMimeVersion' put: 0;
    at: 'HttpQueryModifierFlagsMask' put: 4026531840;
    at: 'HttpQueryOrigUri' put: 34;
    at: 'HttpQueryPragma' put: 17;
    at: 'HttpQueryProxyAuthenticate' put: 41;
    at: 'HttpQueryPublic' put: 8;
    at: 'HttpQueryRawHeaders' put: 21;
    at: 'HttpQueryRawHeadersCrlf' put: 22;
    at: 'HttpQueryReferer' put: 35;
    at: 'HttpQueryRetryAfter' put: 36;
    at: 'HttpQueryServer' put: 37;
    at: 'HttpQueryStatusCode' put: 19;
    at: 'HttpQueryStatusText' put: 20;
    at: 'HttpQueryTitle' put: 38;
    at: 'HttpQueryUri' put: 13;
    at: 'HttpQueryUserAgent' put: 39;
    at: 'HttpQueryVersion' put: 18;
    at: 'HttpQueryWwwAuthenticate' put: 40;
    at: 'HttpStatusAccepted' put: 202;
    at: 'HttpStatusAmbiguous' put: 300;
    at: 'HttpStatusAuthRefused' put: 411;
    at: 'HttpStatusBadGateway' put: 502;
    at: 'HttpStatusBadMethod' put: 405;
    at: 'HttpStatusBadRequest' put: 400;
    at: 'HttpStatusConflict' put: 409;
    at: 'HttpStatusCreated' put: 201;
    at: 'HttpStatusDenied' put: 401;
    at: 'HttpStatusForbidden' put: 403;
    at: 'HttpStatusGatewayTimeout' put: 504;
    at: 'HttpStatusGone' put: 410;
    at: 'HttpStatusMoved' put: 301;
    at: 'HttpStatusNoContent' put: 204;
    at: 'HttpStatusNoneAcceptable' put: 406;
    at: 'HttpStatusNotFound' put: 404;
    at: 'HttpStatusNotModified' put: 304;
    at: 'HttpStatusNotSupported' put: 501;
    at: 'HttpStatusOk' put: 200;
    at: 'HttpStatusPartial' put: 203;
    at: 'HttpStatusPaymentReq' put: 402;
    at: 'HttpStatusProxyAuthReq' put: 407;
    at: 'HttpStatusRedirect' put: 302;
    at: 'HttpStatusRedirectMethod' put: 303;
    at: 'HttpStatusRequestTimeout' put: 408;
    at: 'HttpStatusServerError' put: 500;
    at: 'HttpStatusServiceUnavail' put: 503;
    at: 'HttpVersion' put: 'HTTP/1.0';
    at: 'IcuEscape' put: 2147483648;
    at: 'IcuUsername' put: 1073741824;
    at: 'InternetDefaultFtpPort' put: 21;
    at: 'InternetDefaultGopherPort' put: 70;
    at: 'InternetDefaultHttpPort' put: 80;
    at: 'InternetErrorBase' put: 12000;
    at: 'InternetFirstOption' put: 1;
    at: 'InternetFlagAsync' put: 268435456;
    at: 'InternetFlagDontCache' put: 67108864;
    at: 'InternetFlagExistingConnect' put: 536870912;
    at: 'InternetFlagMakePersistent' put: 33554432;
    at: 'InternetFlagNoAutoRedirect' put: 2097152;
    at: 'InternetFlagNoCloseCallback' put: 4194304;
    at: 'InternetFlagOffline' put: 16777216;
    at: 'InternetFlagPassive' put: 134217728;
    at: 'InternetFlagRawData' put: 1073741824;
    at: 'InternetFlagReload' put: 2147483648;
    at: 'InternetFlagSecure' put: 8388608;
    at: 'InternetFlagsMask' put: 4290772992;
    at: 'InternetHandleTypeConnectFtp' put: 2;
    at: 'InternetHandleTypeConnectGopher' put: 3;
    at: 'InternetHandleTypeConnectHttp' put: 4;
    at: 'InternetHandleTypeFtpFile' put: 7;
    at: 'InternetHandleTypeFtpFileHtml' put: 8;
    at: 'InternetHandleTypeFtpFind' put: 5;
    at: 'InternetHandleTypeFtpFindHtml' put: 6;
    at: 'InternetHandleTypeGopherFile' put: 11;
    at: 'InternetHandleTypeGopherFileHtml' put: 12;
    at: 'InternetHandleTypeGopherFind' put: 9;
    at: 'InternetHandleTypeGopherFindHtml' put: 10;
    at: 'InternetHandleTypeHttpRequest' put: 13;
    at: 'InternetHandleTypeInternet' put: 1;
    at: 'InternetInvalidPortNumber' put: 0;
    at: 'InternetInvalidStatusCallback' put: '-1';
    at: 'InternetKeepAliveDisabled' put: 0;
    at: 'InternetKeepAliveEnabled' put: 1;
    at: 'InternetKeepAliveUnknown' put: -1;
    at: 'InternetLastOption' put: 32;
    at: 'InternetMaxHostNameLength' put: 256;
    at: 'InternetMaxPasswordLength' put: 128;
    at: 'InternetMaxPathLength' put: 1024;
    at: 'InternetMaxPortNumberLength' put: 5;
    at: 'InternetMaxPortNumberValue' put: 65535;
    at: 'InternetMaxProtocolName' put: 'gopher';
    at: 'InternetMaxUrlLength' put: 1032;
    at: 'InternetMaxUserNameLength' put: 128;
    at: 'InternetNoCallback' put: 0;
    at: 'InternetOpenTypeDirect' put: 1;
    at: 'InternetOpenTypePreconfig' put: 0;
    at: 'InternetOpenTypeProxy' put: 3;
    at: 'InternetOptionAsync' put: 30;
    at: 'InternetOptionAsyncId' put: 15;
    at: 'InternetOptionAsyncPriority' put: 16;
    at: 'InternetOptionAsyncQueueDepth' put: 19;
    at: 'InternetOptionAsyncRequestCount' put: 17;
    at: 'InternetOptionCacheStreamHandle' put: 27;
    at: 'InternetOptionCallback' put: 1;
    at: 'InternetOptionConnectBackoff' put: 4;
    at: 'InternetOptionConnectRetries' put: 3;
    at: 'InternetOptionConnectTimeout' put: 2;
    at: 'InternetOptionContextValue' put: 10;
    at: 'InternetOptionControlReceiveTimeout' put: 6;
    at: 'InternetOptionControlSendTimeout' put: 5;
    at: 'InternetOptionDataReceiveTimeout' put: 8;
    at: 'InternetOptionDataSendTimeout' put: 7;
    at: 'InternetOptionExtendedError' put: 24;
    at: 'InternetOptionGatewayName' put: 14;
    at: 'InternetOptionHandleType' put: 9;
    at: 'InternetOptionKeepConnection' put: 22;
    at: 'InternetOptionMaximumWorkerThreads' put: 18;
    at: 'InternetOptionNameResThread' put: 11;
    at: 'InternetOptionOfflineMode' put: 26;
    at: 'InternetOptionParentHandle' put: 21;
    at: 'InternetOptionPassword' put: 29;
    at: 'InternetOptionReadBufferSize' put: 12;
    at: 'InternetOptionReceiveAllMode' put: 25;
    at: 'InternetOptionReceiveTimeout' put: 6;
    at: 'InternetOptionRequestFlags' put: 23;
    at: 'InternetOptionSecurityCertificate' put: 32;
    at: 'InternetOptionSecurityKeyBitness' put: 31;
    at: 'InternetOptionSendTimeout' put: 5;
    at: 'InternetOptionsMask' put: 0;
    at: 'InternetOptionUsername' put: 28;
    at: 'InternetOptionWorkerThreadTimeout' put: 20;
    at: 'InternetOptionWriteBufferSize' put: 13;
    at: 'InternetPriorityForeground' put: 1000;
    at: 'InternetReqflagAsync' put: 2;
    at: 'InternetReqflagFromCache' put: 1;
    at: 'InternetSchemeDefault' put: 0;
    at: 'InternetSchemeFtp' put: 1;
    at: 'InternetSchemeGopher' put: 2;
    at: 'InternetSchemeHttp' put: 3;
    at: 'InternetSchemeHttps' put: 4;
    at: 'InternetServiceFtp' put: 1;
    at: 'InternetServiceGopher' put: 2;
    at: 'InternetServiceHttp' put: 3;
    at: 'InternetStatusClosingConnection' put: 50;
    at: 'InternetStatusConnectedToServer' put: 21;
    at: 'InternetStatusConnectingToServer' put: 20;
    at: 'InternetStatusConnectionClosed' put: 51;
    at: 'InternetStatusCtlResponseReceived' put: 42;
    at: 'InternetStatusHandleClosing' put: 70;
    at: 'InternetStatusHandleCreated' put: 60;
    at: 'InternetStatusNameResolved' put: 11;
    at: 'InternetStatusReceivingResponse' put: 40;
    at: 'InternetStatusRedirect' put: 110;
    at: 'InternetStatusRequestComplete' put: 100;
    at: 'InternetStatusRequestSent' put: 31;
    at: 'InternetStatusResolvingName' put: 10;
    at: 'InternetStatusResponseReceived' put: 41;
    at: 'InternetStatusSendingRequest' put: 30;
    at: 'LocalInternetAccess' put: 1;
    at: 'MaxGopherAttributeName' put: 128;
    at: 'MaxGopherCategoryName' put: 128;
    at: 'MaxGopherDisplayText' put: 128;
    at: 'MaxGopherHostName' put: 256;
    at: 'MaxGopherLocatorLength' put: 653;
    at: 'MaxGopherSelectorText' put: 256;
    at: 'MinGopherAttributeLength' put: 256;
    at: 'PreConfigInternetAccess' put: 0;
    yourself.
!
   
initializeStructures
    "Private - Initializes the structures for the interface..."

    self
        registerType: 'FILETIME'
        as: #(
            (dwLowDateTime ulong)
            (dwHighDateTime ulong)
        );
        registerType: 'INTERNET_ASYNC_RESULT'
        as: #(
            (dwResult ulong)
            (dwError ulong)
        );
        registerType: 'URL_COMPONENTS'
        as: #(
            (lpszScheme ulong)
            (dwSchemeLength ulong)
            (nScheme ulong)
            (lpszHostName ulong)
            (dwHostNameLength ulong)
            (nPort ulong)
            (lpszUserName ulong)
            (dwUserNameLength ulong)
            (lpszPassword ulong)
            (dwPasswordLength ulong)
            (lpszUrlPath ulong)
            (dwUrlPathLength ulong)
        );
        registerType: 'GOPHER_FIND_DATA'
        as: #(
            (displayString 129 asCString)
            (gopherType ulong)
            (sizeLow ulong)
            (sizeHigh ulong)
            (lastModificationTime 'FILETIME')
            (locator 654 asCString)
        );
        registerType: 'GOPHER_ADMIN_ATTRIBUTE_TYPE'
        as: #(
            (comment ulong)
            (emailAddress ulong)
        );
        registerType: 'GOPHER_MOD_DATE_ATTRIBUTE_TYPE'
        as: #(
            (dateAndTime 'FILETIME')
        );
        registerType: 'GOPHER_TTL_ATTRIBUTE_TYPE'
        as: #(
            (ttl ulong)
        );
        registerType: 'GOPHER_SCORE_ATTRIBUTE_TYPE'
        as: #(
            (score long)
        );
        registerType: 'GOPHER_SCORE_RANGE_ATTRIBUTE_TYPE'
        as: #(
            (lowerBound long)
            (upperBound long)
        );
        registerType: 'GOPHER_SITE_ATTRIBUTE_TYPE'
        as: #(
            (site ulong)
        );
        registerType: 'GOPHER_ORGANIZATION_ATTRIBUTE_TYPE'
        as: #(
            (organization ulong)
        );
        registerType: 'GOPHER_LOCATION_ATTRIBUTE_TYPE'
        as: #(
            (location ulong)
        );
        registerType: 'GOPHER_GEOGRAPHICAL_LOCATION_ATTRIBUTE_TYPE'
        as: #(
            (degreesNorth long)
            (minutesNorth long)
            (secondsNorth long)
            (degreesEast long)
            (minutesEast long)
            (secondsEast long)
        );
        registerType: 'GOPHER_TIMEZONE_ATTRIBUTE_TYPE'
        as: #(
            (zone long)
        );
        registerType: 'GOPHER_PROVIDER_ATTRIBUTE_TYPE'
        as: #(
            (provider ulong)
        );
        registerType: 'GOPHER_VERSION_ATTRIBUTE_TYPE'
        as: #(
            (version ulong)
        );
        registerType: 'GOPHER_ABSTRACT_ATTRIBUTE_TYPE'
        as: #(
            (shortAbstract ulong)
            (abstractFile ulong)
        );
        registerType: 'GOPHER_VIEW_ATTRIBUTE_TYPE'
        as: #(
            (contentType ulong)
            (language ulong)
            (size ulong)
        );
        registerType: 'GOPHER_VERONICA_ATTRIBUTE_TYPE'
        as: #(
            (treeWalk ulong)
        );
        registerType: 'GOPHER_VERONICA_ATTRIBUTE_TYPE'
        as: #(
            (treeWalk ulong asBoolean)
        );
        registerType: 'GOPHER_ASK_ATTRIBUTE_TYPE'
        as: #(
            (questionType ulong)
            (questionText ulong)
        );
        registerType: 'GOPHER_UNKNOWN_ATTRIBUTE_TYPE'
        as: #(
            (text ulong)
        );
        registerType: 'GOPHER_ATTRIBUTE_TYPE'
        as: #(
            (categoryId ulong)
            (attributeId ulong)
            (admin 'GOPHER_ADMIN_ATTRIBUTE_TYPE')
            (modDate 'GOPHER_MOD_DATE_ATTRIBUTE_TYPE')
            (ttl 'GOPHER_TTL_ATTRIBUTE_TYPE')
            (score 'GOPHER_SCORE_ATTRIBUTE_TYPE')
            (scoreRange 'GOPHER_SCORE_RANGE_ATTRIBUTE_TYPE')
            (site 'GOPHER_SITE_ATTRIBUTE_TYPE')
            (organization 'GOPHER_ORGANIZATION_ATTRIBUTE_TYPE')
            (location 'GOPHER_LOCATION_ATTRIBUTE_TYPE')
            (geographicalLocation 'GOPHER_GEOGRAPHICAL_LOCATION_ATTRIBUTE_TYPE')
            (timeZone 'GOPHER_TIMEZONE_ATTRIBUTE_TYPE')
            (provider 'GOPHER_PROVIDER_ATTRIBUTE_TYPE')
            (version 'GOPHER_VERSION_ATTRIBUTE_TYPE')
            (abstract 'GOPHER_ABSTRACT_ATTRIBUTE_TYPE')
            (view 'GOPHER_VIEW_ATTRIBUTE_TYPE')
            (veronica 'GOPHER_VERONICA_ATTRIBUTE_TYPE')
            (ask 'GOPHER_ASK_ATTRIBUTE_TYPE')
            (unknown 'GOPHER_UNKNOWN_ATTRIBUTE_TYPE')
        );
        yourself!
  
newCallBack: callBackName sending: selector to: aReceiver
    "Returns a CallBack object for sending message with selector to aReceiver.
    callBackName must be a valid name defined in callbackTable."

    | tuple aCallBack aMessage parameterTypes |
    tuple := self callbackTable
        detect: [:one| one first = callBackName ]
        ifNone: [ ^self error: 'Invalid callback name.'].

    "check selector arguments with parameterTypes size"
    parameterTypes := tuple at: 2.
    (selector asString select: [:c| c = $: ]) size = parameterTypes size
    ifFalse: [ ^self error: 'Invalid selector (Must have ',parameterTypes size asString,' arguments.' ].

    aMessage := Message new
        receiver: aReceiver;
        selector: selector;
        yourself.
    aCallBack := CallBack
        registerMessage: aMessage
        parameterTypes: parameterTypes
        returnType: (tuple at: 3)
        callingConvention: (tuple size > 3 ifTrue: [ tuple at: 4 ] ifFalse: [#api]).
! !



!WinInetDLL methods !
  
close
	"Free the DLL.
	Reimplemented here for secure Connection releasing."

	InternetSession currentOrNil release.
	^super close
    !
   
ftpCommand: hConnect
    fExpectResponse: fExpectResponse
    dwFlags: dwFlags
    lpszCommand: lpszCommand
    dwContext: dwContext

    "The FtpCommand function issues an arbitrary command on the FTP server referred to by the session handle.
    If the function succeeds, the return value is TRUE. If it fails, an error occurred.
    The application can then use GetLastError to retrieve a specific error code.
    If the error code indicates that the FTP server denied the request to change to a directory, InternetGetLastResponseInfo can be used to determine why."

    <api: FtpCommandA handle boolean ulong struct ulong boolean>
    ^self invalidArgument!
   
ftpCreateDirectory: hConnect
    lpszDirectory: lpszDirectory

    "The FtpCreateDirectory function creates a new directory on the FTP server.
    If succeeds, a value of TRUE is returned.
    If an error occurred, a value of FALSE is returned.
    In this case, an application can call GetLastError to retrieve a specific error code."

    <api: FtpCreateDirectoryA handle struct boolean>
    ^self invalidArgument!
   
ftpDeleteFile: hConnect
    lpszFileName: lpszFileName

    "The FtpDeleteFile function deletes a file stored on the FTP server.
    If the file was deleted successfully, a value of TRUE is returned.
    If an error occurred, a value of FALSE is returned.
    In this case, an application can call GetLastError to retrieve a specific error code."

    <api: FtpDeleteFileA handle struct boolean>
    ^self invalidArgument!
 
ftpFindFirstFile: hConnect
    lpszSearchFile: lpszSearchFile
    lpFindFileData: lpFindFileData
    dwFlags: dwFlags
    dwContext: dwContext

    "The FtpFindFirstFile function begins searching the current directory of the given FTP session.
    File and directory entries are returned to the application in the WIN32_FIND_DATA structure.
    If the directory enumeration was started successfully, a valid handle for the request is returned.
    If an error occurred, NULL is returned. In this case, an application can call GetLastError to retrieve a specific error code.
    If no matching files can be found, the GetLastError function returns ERROR_NO_MORE_FILES."

    <api: FtpFindFirstFileA handle struct struct ulong ulong ulongReturn>
    ^self invalidArgument!
 
ftpGetCurrentDirectory: hConnect
    lpszCurrentDirectory: lpszCurrentDirectory
    lpdwCurrentDirectory: lpdwCurrentDirectorySize

    "The FtpGetCurrentDirectory function retrieves the current directory for the specified FTP session.
    If the function succeeds, the return value is TRUE. If it fails, an error occurred.
    The application can then use GetLastError to retrieve a specific error code.
    If the error code indicates that the FTP server denied the request to change to a directory, InternetGetLastResponseInfo can be used to determine why."

    <api: FtpGetCurrentDirectoryA handle struct struct boolean>
    ^self invalidArgument!
  
ftpGetFile: hConnect
    lpszRemoteFile: lpszRemoteFile
    lpszNewFile: lpszNewFile
    fFailIfExists: fFailIfExists
    dwFlagsAndAttributes: dwFlagsAndAttributes
    dwFlags: dwFlags
    dwContext: dwContext

    "The FtpGetFile function retrieves a file from the FTP server and stores it under the specified filename, creating a new local file in the process.
    If the file was retrieved successfully, a value of TRUE is returned, if an error occurred, a value of FALSE is returned.
    In this case, an application can call GetLastError to retrieve a specific error code."

    <api: FtpGetFileA handle struct struct boolean ulong ulong ulong boolean>
    ^self invalidArgument!
  
ftpOpenFile: hConnect
    lpszFileName: lpszFileName
    dwAccess: dwAccess
    dwFlags: dwFlags
    dwContext: dwContext

    "The FtpOpenFile function initiates access to a remote file for writing or reading."

    <api: FtpOpenFileA handle struct ulong ulong ulong ulongReturn>
    ^self invalidArgument!

ftpPutFile: hConnect
    lpszLocalFile: lpszLocalFile
    lpszNewRemoteFile: lpszNewRemoteFile
    dwFlags: dwFlags
    dwContext: dwContext

    "The FtpPutFile function stores a file on the FTP server.
    If the file was stored successfully, a value of TRUE is returned.
    If an error occurred, a value of FALSE is returned. In this case, an application can call GetLastError to retrieve a specific error code."

    <api: FtpPutFileA handle struct struct ulong ulong boolean>
    ^self invalidArgument!
 
ftpRemoveDirectory: hConnect
    lpszDirectory: lpszDirectory

    "The FtpRemoveDirectory function removes a directory on the FTP server.
    If succeeds, a value of TRUE is returned.
    If an error occurred, a value of FALSE is returned.
    In this case, an application can call GetLastError to retrieve a specific error code."

    <api: FtpRemoveDirectoryA handle struct boolean>
    ^self invalidArgument!
   
ftpRenameFile: hConnect
    lpszExisting: lpszExisting
    lpszNew: lpszNew

    "The FtpRenameFile function renames a file stored on the FTP server.
    If the file was reneamed successfully, a value of TRUE is returned.
    If an error occurred, a value of FALSE is returned.
    In this case, an application can call GetLastError to retrieve a specific error code."

    <api: FtpRenameFileA handle struct struct boolean>
    ^self invalidArgument!
   
ftpSetCurrentDirectory: hConnect
    lpszDirectory: lpszDirectory

    "The FtpSetCurrentDirectory function is used to change to a different working directory on the FTP server.
    If the function succeeds, the return value is TRUE. If it fails, an error occurred.
    The application can then use GetLastError to retrieve a specific error code.
    If the error code indicates that the FTP server denied the request to change to a directory, InternetGetLastResponseInfo can be used to determine why."

    <api: FtpSetCurrentDirectoryA handle struct boolean>
    ^self invalidArgument!

gopherCreateLocator: lpszHost
    nServerPort: nServerPort
    lpszDisplayString: lpszDisplayString
    lpszSelectorString: lpszSelectorString
    dwGopherType: dwGopherType
    lpszLocator: lpszLocator
    lpdwBufferLength: lpdwBufferLength

    "The GopherCreateLocator function creates a Gopher or Gopher+ locator string from its component parts.
    TRUE is returned if the function was successful.
    If it was not, FALSE is returned.
    In this case, more information can be obtained using GetLastError and InternetGetLastResponseInfo."

    <api: GopherCreateLocatorA struct ushort struct struct ulong struct struct boolean>
    ^self invalidArgument!
  
gopherFindFirstFile: hConnect
    lpszLocator: lpszLocator
    lpszSearchString: lpszSearchString
    lpFindData: lpFindData
    dwFlags: dwFlags
    dwContext: dwContext

    "The GopherFindFirstFile function, when given a Gopher locator and some search criteria, creates a session with the server and locates the requested documents, binary files, index servers, or directory trees.
    If this function is successful, a valid search handle is returned.
    If it is not, a NULL value is returned.
    The GetLastError and InternetGetLastResponseInfo functions can be used to obtain more information."

    <api: GopherFindFirstFileA handle struct struct ulong ulong ulong ulongReturn>
    ^self invalidArgument!
 
gopherGetAttribute: hConnect
    lpszLocator: lpszLocator
    lpszAttributeName: lpszAttributeName
    lpBuffer: lpBuffer
    dwBufferLength: dwBufferLength
    lpdwCharactersReturned: lpdwCharactersReturned
    lpfnEnumerator: lpfnEnumerator
    dwContext: dwContext

    "The GopherGetAttribute function allows an application to retrieve specific attribute information from the server.
    If the request is satisfied, TRUE is returned. If not, FALSE is returned.
    The GetLastError or InternetGetLastResponseInfo functions can be used to retrieve more information about the error."

    <api: GopherGetAttributeA handle struct struct struct ulong struct ulong ulong boolean>
    ^self invalidArgument!

gopherGetLocatorType: lpszLocator
    lpdwGopherType: lpdwGopherType

    "The GopherGetLocatorType function parses a Gopher locator and determines its attributes.
    TRUE is returned if the function was successful.
    If it was not, FALSE is returned.
    In this case, more information can be obtained using GetLastError and InternetGetLastResponseInfo."

    <api: GopherGetLocatorTypeA struct ulong boolean>
    ^self invalidArgument!
   
gopherOpenFile: hConnect
    lpszLocator: lpszLocator
    lpszView: lpszView
    dwFlags: dwFlags
    dwContext: dwContext

    "The GopherOpenFile function begins reading a Gopher data file from a Gopher server.
    If the file cannot be opened, a NULL value is returned.
    The GetLastError or InternetGetLastResponseInfo functions can be used to determine the cause of the error."

    <api: GopherOpenFileA handle struct struct ulong ulong ulongReturn>
    ^self invalidArgument!
 
gopherSendData: hConnect
    lpszLocator: lpszLocator
    lpszBuffer: lpszBuffer
    dwBufferLength: dwBufferLength
    dwNumberOfCharactersToSend: dwNumberOfCharactersToSend
    lpdwNumberOfCharactersSent: lpdwNumberOfCharactersSent
    dwContext: dwContext

    "???."

    <api: GopherSendDataA handle struct struct ulong ulong ulong ulong boolean>
    ^self invalidArgument!
   
httpAddRequestHeaders: hRequest
    lpszHeaders: lpszHeaders
    dwHeadersLength: dwHeadersLength
    dwModifiers: dwModifiers

    "Adds one or more HTTP request headers to the HTTP request handle.
    If the function is successful, TRUE is returned.
    If not, FALSE is returned.
    Extended error information can be retrieved by calling the GetLastError function."

    <api: HttpAddRequestHeadersA handle struct ulong ulong boolean>
    ^self invalidArgument!

httpOpenRequest: hConnect
    lpszVerb: lpszVerb
    lpszObjectName: lpszObjectName
    lpszVersion: lpszVersion
    lpszReferrer: lpszReferrer
    lplpszAcceptTypes: lplpszAcceptTypes
    dwFlags: dwFlags
    dwContext: dwContext

    "Opens an HTTP request handle.
    If the function was successful, a valid (non-NULL) HTTP request handle is returned.
    If not, a NULL value is returned.
    Extended error information can be retrieved by calling the GetLastError function."

    <api: HttpOpenRequestA handle struct struct struct struct ulong ulong ulong ulongReturn>
    ^self invalidArgument!
 
httpQueryInfo: hRequest
    dwInfoLevel: dwInfoLevel
    lpBuffer: lpBuffer
    lpdwBufferLength: lpdwBufferLength
    lpdwIndex: lpdwIndex

    "Queries information about an HTTP request.
    If the function is successful, TRUE is returned.
    If not, FALSE is returned.
    Extended error information can be retrieved by calling the GetLastError function."

    <api: HttpQueryInfoA handle ulong struct struct struct boolean>
    ^self invalidArgument!
 
httpSendRequest: hRequest
    lpszHeaders: lpszHeaders
    dwHeadersLength: dwHeadersLength
    lpOptional: lpOptional
    dwOptionalLength: dwOptionalLength

    "Sends the specified request to the HTTP server.
    If the function is successful, TRUE is returned.
    If not, FALSE is returned.
    Extended error information can be retrieved by calling the GetLastError function."

    <api: HttpSendRequestA handle struct ulong struct ulong boolean>
    ^self invalidArgument!
 
internetCloseHandle: hInternet

    "The InternetCloseHandle function is used to close all Internet handles and free any resources associated with the handle.
    Pending operations on the handle are terminated and any outstanding data is discarded.
    If one thread is blocking a call to the Internet DLL, another thread in the application can call InternetCloseHandle on the Internet handle being used by the first thread to cancel the operation and unblock the first thread.
    If the handle was closed successfully, a value of TRUE is returned.
    If an error occurred, a value of FALSE will be returned.
    In this case, the application can call GetLastError to retrieve a specific error code."

    <api: InternetCloseHandle handle boolean>
    ^self invalidArgument!
 
internetConnect: hInternet
    lpszServerName: lpszServerName
    nServerPort: nServerPort
    lpszUserName: lpszUserName
    lpszPassword: lpszPassword
    dwService: dwService
    dwFlags: dwFlags
    dwContext: dwContext

    "The InternetConnect function opens an FTP, Gopher, or HTTP session for the specified site.
    If the connection is successfully established, a valid handle to the FTP, Gopher, or HTTP URL is returned.
    If the connection fails, NULL is returned.
    The application can then call GetLastError to retrieve a specific error code.
    Applications can also use InternetGetLastResponseInfo to determine why access to the service was denied."

    <api: InternetConnectA handle struct ulong struct struct ulong ulong ulong ulongReturn>
    ^self invalidArgument!
  
internetCrackUrl: lpszUrl
    dwUrlLength: dwUrlLength
    dwFlags: dwFlags
    lpUrlComponents: lpUrlComponents

    "Cracks an URL into its component parts.
    The required components are indicated by the fields in the URL_COMPONENTS structure: if both the input pointer and length fields are 0 then that component is not returned.
    If the input pointer field is NULL, but the length field is non-zero then on output, the pointer field will point at the corresponding component in the lpszUrl string, and the length field will contain the length of the component.
    If both the input pointer and length fields are not zero then the pointer field points to a user-supplied buffer where the component is copied; the length field will be updated with the length of the copied component, minus 1 for the trailing string terminator.
    Returns TRUE if successful.
    The GetLastError function can be used to retrieve extended error information. If no matching files can be found, the GetLastError function returns ERROR_NO_MORE_FILES."

    <api: InternetCrackUrlA struct ulong ulong struct boolean>
    ^self invalidArgument!

internetCreateUrl: lpUrlComponents
    dwFlags: dwFlags
    lpszUrl: lpszUrl
    lpdwUrlLength: lpdwUrlLength

    "Creates an URL from its component parts.
    Returns TRUE if successful.
    The GetLastError function can be used to retrieve extended error information."

    <api: InternetCreateUrlA struct ulong struct ulong boolean>
    ^self invalidArgument!
   
internetErrorDlg: hWnd
    hRequest: hRequest
    dwError: dwError
    dwFlags: dwFlags
    lppvData: lppvData

    "A client can call InternetErrorDlg for one of several error codes to generate UI, and optionally do bookkeeping on the result of the UI.
    Returns: ERROR_SUCCESS, or INTERNET_ERROR_FORCE_RETRY"

    <api: InternetErrorDlg handle handle ulong ulong ulong ulong>
    ^self invalidArgument!

internetFindNextFile: hFind lpvFindData: lpvFindData

    "The InternetFindNextFile function continues a file search from a previous call to FtpFindFirstFile or GopherFindFirstFile.
    If the function succeeds, the return value is TRUE. If it fails, the return value is  FALSE. The GetLastError function can be used to retrieve extended error information.
    If no matching files can be found, the GetLastError function returns ERROR_NO_MORE_FILES."

    <api: InternetFindNextFileA handle struct boolean>
    ^self invalidArgument!
   
internetGetLastResponseInfo: lpdwError
    lpszBuffer: lpszBuffer
    lpdwBufferLength: lpdwBufferLength

    "The InternetGetLastResponseInfo function retrieves error text from the last Win32 Internet function that failed.
    If error text was successfully written to the buffer, a value of TRUE is returned.
    If an error occurred, a value of FALSE is returned.
    In this case, the application can call GetLastError to retrieve a specific error code.
    If the buffer is too small to hold all the error text, GetLastError returns ERROR_BUFFER_OVERFLOW.
    The lpdwBufferLength parameter will contain the minimum buffer size required to return all the error text."

    <api: InternetGetLastResponseInfoA struct struct struct boolean>
    ^self invalidArgument!
   
internetOpen: lpszAgent
    dwAccessType: dwAccessType
    lpszProxy: lpszProxy
    lpszProxyBypass: lpszProxyBypass
    dwFlags: dwFlags

    "The InternetOpen function is the first Win32 Internet function called by an application.
    It tells the Internet DLL to initialize internal data structures and prepare for future calls from the application.
    When the application is finished using the Internet function, it should call InternetCloseHandle to free the HINTERNET and any other resources.
    Returns a valid handle which the application passes on to subsequent Win32 Internet functions.
    If InternetOpen fails, it will return a NULL value.
    The application can then retrieve a specific error code with the GetLastError function."

    <api: InternetOpenA struct ulong struct struct ulong ulongReturn>
    ^self invalidArgument!

internetOpenUrl: hInternetSession
    lpszUrl: lpszUrl
    lpszHeaders: lpszHeaders
    dwHeadersLength: dwHeadersLength
    dwFlags: dwFlags
    dwContext: dwContext

    "The InternetOpenUrl function begins reading an FTP, Gopher, or HTTP Universal Resource Locator (URL).
    If the connection is successfully established, a valid handle to the FTP, Gopher, or HTTP URL is returned.
    If the connection fails, NULL is returned.
    The application can then call GetLastError to retrieve a specific error code.
    Applications can also use InternetGetLastResponseInfo to determine why access to the service was denied."

    <api: InternetOpenUrlA handle struct struct ulong ulong ulong ulongReturn>
    ^self invalidArgument!
   
internetQueryDataAvailable: hInternetFile
    lpdwNumberOfBytesAvailable: lpdwNumberOfBytesAvailable
    dwFlags: dwFlags
    dwContext: dwContext

    "The InternetQueryDataAvailable function queries the amount of data available.
    This function fill lpdwNumberOfBytesAvailable holder with the number of bytes of data that are available to be read immediately by a subsequent call to InternetReadFile.
    If there is currently no data available and the end-of-file has not been reached, the request pends until data becomes available.
    If the function succeeds, the return value is TRUE. If it fails, the return value is  FALSE. The GetLastError function can be used to retrieve extended error information.
    If no matching files can be found, the GetLastError function returns ERROR_NO_MORE_FILES."

    <api: InternetQueryDataAvailable handle struct ulong ulong boolean>
    ^self invalidArgument!

internetQueryOption: hInternet
    dwOption: dwOption
    lpBuffer: lpBuffer
    lpdwBufferLength: lpdwBufferLength

    "The InternetQueryOption function queries an Internet option on the specified handle.
    If the function succeeds, the return value is TRUE. If it fails, the return value is  FALSE.
    The GetLastError function can be used to retrieve extended error information."

    <api: InternetQueryOption handle ulong struct struct boolean>
    ^self invalidArgument!
  
internetReadFile: hInternetFile
    lpBuffer: lpBuffer
    dwNumberOfBytesToRead: dwNumberOfBytesToRead
    lpdwNumberOfBytesRead: lpdwNumberOfBytesRead

    "Reads data from a handle opened by the InternetOpenUrl, FtpOpenFile, GopherOpenFile, or HttpOpenRequest APIs.
    If the function succeeds, the return value is TRUE. If it fails, the return value is  FALSE.
    The application can then call GetLastError to retrieve a specific error code.
    Applications can also use InternetGetLastResponseInfo to determine why access to the service was denied."

    <api: InternetReadFile handle struct ulong struct boolean>
    ^self invalidArgument!
 
internetSetFilePointer: hInternetFile
    lDistanceToMove: lDistanceToMove
    pReserved: pReserved
    dwMoveMethod: dwMoveMethod

    "(Not currently implemented in this API version)
    InternetSetFilePointer sets a file position for InternetReadFile.
    It is a synchronous call, however subsequent calls to InternetReadFile may block or return pending if the data is not available from the cache and the server does not support random access.
    Returns -1 on failure, else the current file position.
    dwMoveMethod can be:
        FileBegin    The starting point is zero or the beginning of the file. If FileBegin is specified, DistanceToMove is interpreted as an unsigned location for the new file pointer.
        FileCurrent    The current value of the file pointer is the starting     point.
        FileEnd    The current end-of-file position is the starting point. This method will fail if the content length is unknown.
    found in OperatingSystemConstants
    "

    <api: InternetSetFilePointer handle long ulong ulong long>
    ^self invalidArgument!

internetSetOption: hInternet
    dwOption: dwOption
    lpBuffer: lpBuffer
    dwBufferLength: dwBufferLength

    "The InternetSetOption function sets an Internet option on the specified handle.
    If the function succeeds, the return value is TRUE. If it fails, the return value is  FALSE.
    The GetLastError function can be used to retrieve extended error information."

    <api: InternetSetOption handle ulong struct ulong boolean>
    ^self invalidArgument!

internetSetStatusCallback: hInternet
    lpfnInternetCallback: lpfnInternetCallback

    "The InternetSetStatusCallback function sets up a callback function that Win32 Internet functions can call when progress is made during an operation.
    This function returns the previously defined status callback function.
    It returns NULL if there was no previously defined status callback,
     or InternetInvalidStatusCallback (-1) if the callback is not valid."

    <api: InternetSetStatusCallback handle ulong long>
    ^self invalidArgument!
 
internetSetStatusCallback: hInternet
    sending: selector
    to: aReceiver

    "The InternetSetStatusCallback function sets up a callback function that Win32 Internet functions can call when progress is made during an operation.
    This function returns the previously defined status callback function address.
    It returns NULL if there was no previously defined status callback,
     or InternetInvalidStatusCallback (-1) if the callback is not valid."

    | callBack result |
    callBack := self class
        newCallBack: #internetStatusCallback
        sending: selector
        to: aReceiver.
    result := self
            internetSetStatusCallback: hInternet
            lpfnInternetCallback: callBack.
    result = InternetInvalidStatusCallback ifTrue: [ callBack release ].
    ^result!
  
internetWriteFile: hInternetFile
    lpBuffer: lpBuffer
    dwNumberOfBytesToWrite: dwNumberOfBytesToWrite
    lpdwNumberOfBytesWritten: lpdwNumberOfBytesWritten

    "The InternetWriteFile function writes data to an open Internet file.
    If the function succeeds, the return value is TRUE. If it fails, the return value is  FALSE.
    The application can then call GetLastError to retrieve a specific error code.
    Applications can also use InternetGetLastResponseInfo to determine why access to the service was denied."

    <api: InternetWriteFile handle struct ulong ulong boolean>
    ^self invalidArgument!
 
isGopherAsk: aType
    " Macro, Detection - Returns true for a gopher Ask type. "

    ^(aType bitAnd: GopherTypeAsk) ~= 0!
  
isGopherDirectory: aType
    " Macro, Detection - Returns true for a gopher directory type. "

    ^(aType bitAnd: GopherTypeDirectory) ~= 0!

isGopherError: aType
    " Macro, Detection - Returns true for a gopher error type. "

    ^(aType bitAnd: GopherTypeError) ~= 0!

isGopherFile: aType
    " Macro, Detection - Returns true for a gopher file type. "

    ^(aType bitAnd: GopherTypeFileMask) ~= 0!
   
isGopherIndexServer: aType
    " Macro, Detection - Returns true for a gopher IndexServer type. "

    ^(aType bitAnd: GopherTypeIndexServer) ~= 0!
  
isGopherPhoneServer: aType
    " Macro, Detection - Returns true for a gopher phoneServer type. "

    ^(aType bitAnd: GopherTypeCso) ~= 0!
  
isGopherPlus: aType
    " Macro, Detection - Returns true for a gopher Plus type. "

    ^(aType bitAnd: GopherTypeGopherPlus) ~= 0!
 
isGopherTelnet: aType
    " Macro, Detection - Returns true for a gopher TelnetSession type. "

    ^(aType bitAnd: GopherTypeTelnet) ~= 0!
  
isGopherTelnetSession: aType
    " Macro, Detection - Returns true for a gopher TelnetSession type. "

    ^(aType bitAnd: GopherTypeTelnet) ~= 0!
   
isGopherTn3270Session: aType
    " Macro, Detection - Returns true for a gopher Tn3270Session type. "

    ^(aType bitAnd: GopherTypeTn3270) ~= 0!
   
isGopherTypeKnown: aType
    " Macro, Detection - Returns true for a gopher of known type. "

    ^(aType bitAnd: GopherTypeUnknown) = 0! !
